/*****************************************************
 *
 * BLAZER (C) Finger @ Fingers elektrische Welt
 *
 * www.fingers-welt.de
 * finger@nwn.de
 *
 * ImageCraft-C
 * AT90S8585 @ 8 MHz
 *
 * TAB-Space 4
 *
 ****************************************************/
#include <io8535v.h>
#include <macros.h>

// I2C-Leitungen an Port D
#define SDA 0x08
#define SCL 0x10

// Verzgerungszeiten fr I2C
#define uDLY                  40         // Verzgerungszeit zwischen Bitwechseln
#define rDLY                  2          // Verzgerungszeit beim Lesen

// Generelle Definitionen
#define FALSE				  0
#define TRUE				  1

// Adressen der I2C-IO-Expander
#define Expander1			  0x70
#define Expander2			  0x72

// Timeouts fr die Motoren *10ms
#define Mundstueck_Timeout    100
#define Gurt_Timeout          200
#define Heber_Timeout         50
#define Wischer_Timeout       100
#define Brenner_Timeout       150

// Diverse Timeouts
#define Gas_Timeout           250
#define Hochspannung_Timeout  300

// Eingangsdefinitionen der I2C-Sensorphalanx
#define Mundstueck_drin             0x0001 // Geprft
#define Mundstueck_draussen         0x0002 // Geprft

#define Arm_frei                    0x0004 // Geprft
#define Arm_Kommutierung            0x0008 // Geprft
#define Brenner_drin                0x0010 // Geprft
#define Brenner_draussen            0x0020 // Geprft

#define Kopf_oben                   0x0080  // Geprft
#define Kopf_unten                  0x0040  // Geprft

#define Wischer_drin                0x0200  // Geprft

#define Taster_Reinigen             0x0100  // Geprft
#define Taster_Rauchen              0x0400  // Geprft
#define Taster_Ein                  0x0800  // Geprft

// Ausgangsdefinitionen der IO-Phaserbnke
#define LED_Ein                     0x01    // Port C // Geprft
#define abc                         0x02    // Port C
#define abcd                        0x04    // Port C

#define Mundstueck_Dir              0x40    // Port C Geprft
#define Mundstueck_Ein              0x80    // Port A Geprft

#define Arm_Dir                     0x10    // Port C Geprft
#define Arm_Ein                     0x20    // Port C Geprft
#define Arm_Halten                  0x80    // Port C Geprft

#define Kopf_Dir                    0x40    // Port A Geprft
#define Kopf_Ein                    0x20    // Port A Geprft
#define Zuender_Dir                 0x10    // Port A Geprft
#define Zuender_Ein                 0x08    // Port A Geprft

#define Reiniger_Ein                0x04    // Port A Geprft
#define Reiniger_Dir                0x02    // Port A Geprft

#define Zwoelf                      0x01    // Port B Geprft
#define Pumpe                       0x02    // Port B Geprft
#define Laser                       0x04    // Port B Geprft
#define Hochspannung                0x08    // Port B Geprft
#define Ventil                      0x10    // Port B Geprft

#define Speaker                     0x20    // Port D Geprft
#define Aux2                        0x40    // Port D Geprft
#define Aux3                        0x80    // Port D Geprft

// Speicheradressen
#define Adresse_Zuender             0x01    // Flag fr den aktivierten Hochspannungsznder

// Betriebsspannung
#define Minimalenergie              4100    // Minimale Betriebsspannung in mV

// Laserblinken
#define Laserein                    10
#define Laseraus                    90

// Variablen
volatile char Taste = 0;					// Empfangenes Zeichen via RS232
volatile char msTicks = 0;                  // Zhler fr den Timerinterrupt
volatile long Timerticks = 0;				// Systemuhr
volatile long Delayticks = 0;				// Fr Warteschleifen
volatile long Spannticks = 0;               // Zur Kommutierungsdetektion

// Aktuelles Geschehen
volatile unsigned int Eingang = 0;          // Eingangsvariablen
volatile char EingangL;
volatile char EingangH;
volatile char Laserblink = FALSE;
volatile char Lasertimer = 0;

// Eingeschaltete Subeinheiten
char FPumpe          = FALSE;
char FVentil         = FALSE;
char FZuender        = FALSE;
char FLaser          = FALSE;
char FWischer        = FALSE;
char FSpeaker        = FALSE;
char FHeber          = FALSE;

// Aktivierte, kognitive Funktionen
char KMundsteck      = FALSE;
char KGurt           = FALSE;
char KKopf           = FALSE;
char KWischer        = FALSE;
char KBrenner        = FALSE;

// Erfolgsmeldung
char Erfolg          = FALSE;
char I2CAcknowledge  = FALSE;
char Absturzflag     = FALSE;

// Geruscherzeugung
char Sound           = 0;

// Diverses
long wert             = 0;

/***************************************************
 *
 * Schaltet den Sound ein
 *
 ***************************************************/
void Sound_ein (void)
{
 TCCR1B=0x04;       // Timer 1 starten
 DDRD |= Speaker;   // Port als Ausgang
}

/***************************************************
 *
 * Schaltet den Sound aus
 *
 ***************************************************/
void Sound_aus (void)
{
 TCCR1B &= 0xF8;    // 1111.1000, Timer 1 anhalten
 DDRD &= ~Speaker;  // Port als Eingang
}

/***************************************************
 *
 * Macht die Ports klar
 *
 ***************************************************/
void port_init(void)
{                           // DDR = 1 : Ausgang !
 PORTA = 0xFF;
 DDRA  = 0xFE;              // Alles Ausgang, bis auf A0

 PORTB = 0x00;              // Alles auf Null
 DDRB  = 0x1F;              // 0001.1111

 PORTC = 0xFF;              // 1111.1111 Alles auf High
 DDRC  = 0xFF;              // 1111.1111 Alles auf Ausgang schalten

 PORTD = 0x1F;              // 0001.1111
 DDRD  = 0xF8;              // 1111.1000 Obere 3 auf Ausgang schalten

 // Armgurt auf Haltestrom zurckschalten
 PORTC &= ~Arm_Halten;

}

/***************************************************
 *
 * Macht Timer 0 klar, freilaufend mit 100 KHz
 *
 ***************************************************/
void timer0_init (void)
{
 TCCR0 = 0x00;  // stop
 TCNT0 = 0x00;  // set count
 TCCR0 = 0x03;  // start timer, prescaler 64
}

/***************************************************
 *
 * Macht Timer 1 klar, freilaufend
 * Toggle an OCR1
 *
 ***************************************************/
void timer1_init(void)
{
 TCCR1A=0x40;
 TCCR1B=0x04;

 TCNT1H=0x00;
 TCNT1L=0x00;

 OCR1AH=0xFF;
 OCR1AL=0xFE;

 OCR1BH=0x00;
 OCR1BL=0x00;

 Sound_aus ();

}

/***************************************************
 *
 * Timer1-Overflow ISR
 *
 ***************************************************/
#pragma interrupt_handler timer1_ovf_isr:9
void timer1_ovf_isr(void)
{
 TCNT1H = 0xFF; //reload counter high value
 TCNT1L = Sound; //reload counter low value
}

/***************************************************
 *
 * Macht den UART klar, 38400 BPS
 *
 ***************************************************/
void uart0_init(void)
{
 UCR  = 0x00; //disable while setting baud rate
 UBRR = 0x0C; //set baud rate 38400 @ 8 MHz
 UCR  = 0xD8; //enable
}

/***************************************************
 *
 * Timer-0 ISR
 * Erhht alle 10 ms den Systemticker
 *
 ***************************************************/
 #pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
 TCNT0 = 0x00; 			   					//reload counter value
 msTicks++;
 if (msTicks == 10)
  {
   Timerticks++; 						 		// Uhr hochzhlen
   msTicks=0;

   Lasertimer++;
   if (Lasertimer == Laserein) Laserblink = FALSE;
   if (Lasertimer == Laseraus)
    {
     Laserblink = TRUE;
     Lasertimer = 0;
    }

  }
}

/***************************************************
 *
 * UART-Receive ISR
 *
 ***************************************************/
#pragma interrupt_handler uart0_rx_isr:12
void uart0_rx_isr(void)
{
 char Junk;

 // Empfangenes Byte verwerfen, wenn das letzte noch nicht verwurstet wurde
 if (Taste != 0) Junk = UDR; else Taste = UDR;
}

/***************************************************
 *
 * UART-Transmit-ISR
 *
 ***************************************************/
#pragma interrupt_handler uart0_tx_isr:14
void uart0_tx_isr(void)
{
}

/***************************************************
 *
 * Macht den ADC klar
 *
 ***************************************************/
void adc_init(void)
{
 ADCSR = 0x00; //disable adc
 ADMUX = 0x00; //select adc input 0
 ADCSR = 0xE1;
}

/***************************************************
 *
 * Sendet ein Byte via UART
 *
 ***************************************************/
void UART_SendByte (char Data)
{
    while(!(USR & 0x20));
    UDR=Data;  // Und raus mit dem Scheiss
}

/***************************************************
 *
 * Sendet eine Zeichenkette via UART
 *
 ***************************************************/
void Uart_SendString (const char* Data)
{
  do
   UART_SendByte (*(Data++));
  while (*Data != '\0');
}

/***************************************************
 *
 * Sendet eine Zahl im Klartext via UART
 *
 ***************************************************/
void UART_SendDec (unsigned int Data)
{
      char         c;
      unsigned int i;
      char         flag=0;

      UART_SendByte (0x20);

      for (i=10000;i!=0;i/=10)
      {
          c=Data/i;
          Data%=i;
          if(c!=0||flag==1||i==1)
          {
              flag=1;
              c+=0x30;
              UART_SendByte (c);
          }
          if(flag==0 && i<=100)
          {
              UART_SendByte (0x20);
          }
      }
}

/***************************************************
 *
 * Liest ein Byte aus dem EEPROM
 *
 ***************************************************/
char EERead (char address)
{
    while (EECR & 0x02);
    EEAR = address;
    EECR = 0x01;
    return EEDR;
}

/***************************************************
 *
 * Schreibt ein Byte ins EEPROM
 *
 ***************************************************/
void EEWrite (char address, char data)
{
    while (EECR & 0x02);
    EEAR = address;
    EEDR = data;
    EECR = 0x04;
    EECR = 0x02;
}

/***************************************************
 *
 * Wartet Ticks * 10ms
 *
 ***************************************************/
void Delay (unsigned int Ticks)
  {
   Delayticks = Timerticks;
   while ( (Delayticks+Ticks) > Timerticks);
  }

/******************************
 *
 * Wartet einige Mikrosekunden
 *
 ******************************/
void DelayUs (int wert)
{
 int z,i = 0;
 for (i=0; i<wert; i++) z = i+1;
}

/***************************************************
 *
 * Generiert einfache Soundeffekte
 *
 ***************************************************/
void Soundeffekt (char Nummer)
{
 char i = 0;

 Delay (30);

 if (Nummer == 1)
  {
    Sound = 0;
    Sound_ein ();
    Delay (10);
    for (i=1; i < 200; i++)
     {
      Sound = i;
      Delay (2);
     }
    Sound_aus ();
  } // Von if (Nummer == 1)

 if (Nummer == 2)
  {
    Sound = 200;
    Sound_ein ();
    Delay (10);
    for (i=200; i>1; i--)
     {
      Sound = i;
      Delay (2);
     }
    Sound_aus ();
  } // Von if (Nummer == 2)

 if (Nummer == 3)
  {
    Sound = 240;
    Sound_ein ();
    Delay (30);
    Sound_aus ();
  } // Von if (Nummer == 3)

 if (Nummer == 4)
  {
    Sound = 250;
    Sound_ein ();
    Delay (10);
    Sound = 240;
    Delay (10);
    Sound = 250;
    Delay (10);
    Sound = 240;
    Delay (10);
    Sound = 250;
    Delay (10);
    Sound_aus ();
  } // Von if (Nummer == 4)

 if (Nummer == 5)
  {
    Sound = 250;
    Sound_ein ();
    Delay (10);
    Sound = 200;
    Delay (10);
    Sound = 250;
    Delay (10);
    Sound = 200;
    Delay (10);
    Sound = 250;
    Delay (10);
    Sound_aus ();
  } // Von if (Nummer == 5)


}

/************************
 *
 * Schaltet SDA auf High
 *
 ************************/
void SDA_High (void)
{
  PORTD |= SDA;																										// SDA = 1;
}

/************************
 *
 * Schaltet SDA auf Low
 *
 ************************/
void SDA_Low (void)
{
  PORTD &= ~SDA;																										// SDA = 0;
}

/************************
 *
 * Schaltet SDA auf High
 *
 ************************/
void SCL_High (void)
{
  PORTD |= SCL;																										// SCL = 1;
}

/************************
 *
 * Schaltet SCL auf Low
 *
 ************************/
void SCL_Low (void)
{
  PORTD &= ~SCL;																										// SCL = 0;
}

/***********************************************************************************
 *
 * Schaltet SDA und SCL als Ausgang und setzt beide auf 1
 *
 ***********************************************************************************/
void i2c_init (void)
 {
  DDRD |= SDA;																										// PortD.3 als Ausgang (1)
  DDRD |= SCL;																										// PortD.4 als Ausgang (1)

  SDA_High ();
  SCL_High ();
 }

/*************************************************************************************
 *
 * Schreibt ein Byte via i2C, erzeugt kein Start/Stop, wertet ACK vom Slave nicht aus
 *
 *************************************************************************************/
char i2c_writebyte (unsigned char tbyte)
 {
  unsigned char i;
  char Ack = FALSE;

  for (i=0; i<8; i++)
   {
     if (tbyte>127) SDA_High (); else SDA_Low ();                           // SDA bedienen
     DelayUs (uDLY);                                            // kurze Pause
     SCL_High (); DelayUs (uDLY);                                     // SCL = 1 + Pause
     SCL_Low (); DelayUs (uDLY);                                     // SCL = 0 + Pause
     tbyte <<= 1;
    }

  // 9. Bit erzeugen, damit der Slave das Acknowledge loswerden kann
  DDRD &= ~SDA;														// PortD.3 als Eingang (0)
  PORTD &= ~SDA;													// Pullup abschalten

  SCL_High ();DelayUs (uDLY);                                      // SCL = 1

  Ack = !(PIND & SDA);                                             // Acknowledge reinholen

  SCL_Low (); DelayUs (uDLY);                                      // SCL = 0 + Pause um ACK abzuwarten

  SCL_Low ();                                                      // SCL = 0

  DDRD |= SDA;													   // PortD.3 als Ausgang (1)

  return (Ack);

 }

/***********************************************************************************
 *
 * Liest ein Byte via i2C, erzeugt bei Bedarf ein Acknowledge
 *
 ***********************************************************************************/
unsigned char i2c_readbyte (char Acknowledge)
 {
  unsigned char i, tbyte = 0;

  DDRD &= ~SDA;																										// PortD.3 als Eingang (0)

  for (i=0; i<8; i++)                                             // Bits rausschieben
   {
     tbyte <<= 1;
     if (PIND & SDA) tbyte |= 1;
     DelayUs (rDLY);
     SCL_High (); DelayUs (rDLY);
     SCL_Low (); DelayUs (rDLY);
   }

  // 9. Bit erzeugen
  DDRD |= SDA;																										// PortD.3 als Ausgang (1)

  if (Acknowledge) SDA_Low (); else SDA_High ();
  DelayUs (rDLY);                                                  // Pause
  SCL_High (); DelayUs (rDLY);                                           // SCL = 1 + Pause
  SCL_Low (); DelayUs (rDLY);                                           // SCL = 0 + Pause
  SDA_High ();DelayUs (rDLY);                                            // SDA = 1 + Pause

  return (tbyte);
}

/***********************************************************************************
 *
 * Liest ein Byte via i2C an Adresse "adress", erzeugt Start/Stop
 *
 ***********************************************************************************/
unsigned char i2c_read (unsigned char deviceaddr)
 {
  unsigned char tbyte;
  char Ack = FALSE;

  // Start-Condition erzeugen
  SDA_High (); SCL_High (); DelayUs (rDLY);                      // SDA + SCL = 1 + Pause
  SDA_Low (); DelayUs (rDLY);                                    // SDA = 0 + Pause
  SCL_Low (); DelayUs (rDLY);                                    // SCL = 0 + Pause

  // Slave zum Lesen adressieren
  tbyte = deviceaddr+1;                                          // Leseadresse setzen
  Ack = i2c_writebyte (tbyte);                                   // Adresse senden

  tbyte = i2c_readbyte (FALSE);                                  // Ein Byte lesen

  // Stop-Condition erzeugen
  SDA_Low (); DelayUs (rDLY);                                    // SDA = 0 + Pause
  SCL_High (); DelayUs (rDLY);                                   // SCL = 1 + Pause
  SDA_High (); DelayUs (rDLY);                                   // SCL = 1 + Pause

  I2CAcknowledge = Ack;

  return (tbyte);
 }

/***************************************************
 *
 * Int0-ISR mit fallender Flanke
 *
 ***************************************************/
#pragma interrupt_handler int0_isr:2
void int0_isr (void)
{
 // Neue Eingangsdaten einladen via I2C
 EingangH = i2c_read (Expander1);
 EingangL = i2c_read (Expander2);
 Eingang = (EingangH << 8) | EingangL;
}


//--------------------- Motorfunktionen -----------------------------------------------------

/***************************************************
 *
 * Schaltet das Gebritzel ein FRSSSSZZZZZSSZ
 *
 ***************************************************/
void Hochspannung_ein (void)
{
 EEWrite (Adresse_Zuender,TRUE);      // Flag setzen
 PORTB |= Hochspannung;               // Und ein
}

/***************************************************
 *
 * Schaltet das Gebritzel aus
 *
 ***************************************************/
void Hochspannung_aus (void)
{
  EEWrite (Adresse_Zuender,FALSE);    // Flag rcksetzen
  PORTB &= ~Hochspannung;             // Und aus
}

/***************************************************
 *
 * Fhrt das Mundstck aus (TRUE wenn alles OK)
 *
 ***************************************************/
char Mundstueck_ausfahren (void)
{
 // Isses schon draussen ?
 if (!(Eingang & Mundstueck_draussen)) return (TRUE);

 // Offensichtlich nicht......
 PORTC |= Mundstueck_Dir;                   // Richtung festlegen
 PORTA &= ~Mundstueck_Ein;                  // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Mundstueck_draussen) && (Timerticks < Mundstueck_Timeout) );

//  PORTC &= ~Mundstueck_Dir;               // Richtung festlegen
// Delay (1);
 PORTA |= Mundstueck_Ein;                   // Motor abschalten

 return (Timerticks < Mundstueck_Timeout);
}

/***************************************************
 *
 * Fhrt das Mundstck ein (TRUE wenn alles OK)
 *
 ***************************************************/
char Mundstueck_einfahren (void)
{
 // Isses schon draussen ?
 if (!(Eingang & Mundstueck_drin)) return (TRUE);

 // Offensichtlich nicht......
 PORTC &= ~Mundstueck_Dir;                  // Richtung festlegen
 PORTA &= ~Mundstueck_Ein;                  // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Mundstueck_drin) && (Timerticks < Mundstueck_Timeout) );

 PORTC |= Mundstueck_Dir;                   // Richtung festlegen
 Delay (1);
 PORTA |= Mundstueck_Ein;                   // Motor abschalten

 return (Timerticks < Mundstueck_Timeout);
}

/***************************************************
 *
 * Hlt den Arm des Delinquenten fest
 *
 ***************************************************/
char Arm_spannen (void)
{
 unsigned int TempEingang = 0;
 char Fertig = FALSE;

 #define Spanntimeout 20

 PORTC |= Arm_Dir;                          // Richtung festlegen
 PORTC |= Arm_Halten;                       // Fahrstrom einschalten
 PORTC &= ~Arm_Ein;                         // Motor einschalten

 // Warten, bis keine Kommutierungsimpulse mehr kommen oder die Zeit berschritten wurde
 Timerticks = 0;
 Delayticks = 0;
 Spannticks = 0;

 TempEingang = (Eingang & Arm_Kommutierung);

 while (!Fertig && (Timerticks < Gurt_Timeout) )
  {

   if ( TempEingang != (Eingang & Arm_Kommutierung) )
    {
     // Signal hat sich gendert
     TempEingang = (Eingang & Arm_Kommutierung);
     Spannticks = 0;
    } else
    { // Signal hat sich nicht gendert
      if (Timerticks != Delayticks)
       {
        Delayticks = Timerticks;
        Spannticks++;
        if (Spannticks > Spanntimeout) Fertig = TRUE;
       }
    }

  } // Von While....

 PORTC &= ~Arm_Halten;                   // Auf Haltestrom zurckschalten
 if (!Fertig)  PORTC |= Arm_Ein;         // Motor abschalten im Fehlerfall

 return (Fertig);
}


/***************************************************
 *
 * Gibt den Arm des Delinquenten frei
 *
 ***************************************************/
char Arm_freigeben (void)
{

 // Arm schon frei ?
 if (!(Eingang & Arm_frei)) return (TRUE);

 PORTC &= ~Arm_Dir;         // Richtung festlegen
 PORTC |= Arm_Halten;       // Fahrstrom einschalten
 PORTC &= ~Arm_Ein;         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Arm_frei) && (Timerticks < Gurt_Timeout) );

 if (Timerticks >= Gurt_Timeout)
  {
   PORTC &= ~Arm_Halten;      // Auf Haltestrom zurckschalten
   PORTC |= Arm_Ein;          // Motor abschalten
   return (FALSE);
  }

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Arm_frei) && (Timerticks < Gurt_Timeout) );

 if (Timerticks >= Gurt_Timeout)
  {
   PORTC &= ~Arm_Halten;      // Auf Haltestrom zurckschalten
   PORTC |= Arm_Ein;          // Motor abschalten
   return (FALSE);
  }

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Arm_frei) && (Timerticks < Gurt_Timeout) );

 PORTC &= ~Arm_Halten;      // Auf Haltestrom zurckschalten
 PORTC |= Arm_Ein;          // Motor abschalten

 return (Timerticks < Gurt_Timeout);

}

/***************************************************
 *
 * Kopf runter
 *
 ***************************************************/
char Kopf_senken (void)
{

 Uart_SendString (" Runter ");

 // Iss der kleine Scheisser schon unten ?
 if (!(Eingang & Kopf_unten)) return (TRUE);

 PORTB |=Zwoelf;       // 12V-Versorgung einschalten
 Delay (10);
 PORTA &= ~Kopf_Dir;   // Richtung festlegen
 PORTA &= ~Kopf_Ein;   // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Kopf_unten) && (Timerticks < Heber_Timeout) );

 PORTA |= Kopf_Dir;   // Richtung festlegen
 DelayUs (200);
 PORTA |= Kopf_Ein;     // Motor abschalten
 PORTB &=~Zwoelf;       // 12V-Versorgung abschalten

 return (Timerticks < Heber_Timeout);

}

/***************************************************
 *
 * Kopf rauf
 *
 ***************************************************/
char Kopf_heben (void)
{
 char ok = FALSE;

 Uart_SendString (" Rauf ");

 // Iss der kleine Scheisser schon oben ?
 if (!(Eingang & Kopf_oben)) return (TRUE);

 PORTB |=Zwoelf;       // 12V-Versorgung einschalten
 Delay (10);
 PORTA |= Kopf_Dir;    // Richtung festlegen
 PORTA &= ~Kopf_Ein;   // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Kopf_oben) && (Timerticks < Heber_Timeout) );

 if (Timerticks >= Heber_Timeout)
  {
   PORTA |= Kopf_Ein;    // Motor abschalten
   PORTB &=~Zwoelf;      // 12V-Versorgung abschalten
   return (FALSE);
  }

 PORTA |= Kopf_Ein;      // Motor abschalten
 PORTB &=~Zwoelf;        // 12V-Versorgung abschalten

 return (TRUE);

}

/***************************************************
 *
 * Reinigt den Kopf
 *
 ***************************************************/
char wischen (void)
{
   PORTA &= ~Reiniger_Dir;       // Richtung festlegen
   PORTA &= ~Reiniger_Ein;       // Wischer einschalten

   // Ist der Sensor aktiv ?
   Timerticks = 0;
   if (!(Eingang & Wischer_drin))
   while ( (!(Eingang & Wischer_drin)) && (Timerticks < Wischer_Timeout) );
   if (Timerticks >= Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }

   // Warten, bis er wieder drin ist
   Timerticks = 0;
   while ( (Eingang & Wischer_drin) && (Timerticks < Wischer_Timeout) );

   if (Timerticks > Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }

   // Warten, bis er wieder draussen ist
   Timerticks = 0;
   while ( !(Eingang & Wischer_drin) && (Timerticks < Wischer_Timeout) );

   if (Timerticks > Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }

   // Warten, bis er wieder drin ist
   Timerticks = 0;
   while ( (Eingang & Wischer_drin) && (Timerticks < Wischer_Timeout) );

   if (Timerticks > Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }


   PORTA |= Reiniger_Dir;       // Richtung umschalten und damit den Motor bremsen
   DelayUs (500);

   if (Timerticks > Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }

   while ( !(Eingang & Wischer_drin)); // Warten, bis er wieder draussen ist

   PORTA |= Reiniger_Ein;       // Wischer abschalten

   return (Timerticks < Wischer_Timeout);

}

/***************************************************
 *
 * Fhrt den Znder aus
 *
 ***************************************************/
char zuender_ausfahren (void)
{

 // Isses schon draussen ?
 if (!(Eingang & Brenner_draussen)) return (TRUE);

 PORTB |=Zwoelf;                                // 12V-Versorgung einschalten
 Delay (10);

 // Offensichtlich nicht......
 PORTA |= Zuender_Dir;                         // Richtung festlegen
 PORTA &= ~Zuender_Ein;                         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Brenner_draussen) && (Timerticks < Brenner_Timeout) );

 Delay (20);
 PORTA |= Zuender_Ein;                         // Motor abschalten

 PORTB &= ~Zwoelf;                             // 12V-Versorgung abschalten

 return (Timerticks < Brenner_Timeout);

}

/***************************************************
 *
 * Fhrt den Znder ein
 *
 ***************************************************/
char zuender_einfahren (void)
{
 // Isses schon drin ?
 if (!(Eingang & Brenner_drin)) return (TRUE);

 PORTB |=Zwoelf;                                // 12V-Versorgung einschalten
 Delay (10);

 // Offensichtlich nicht......
 PORTA &= ~Zuender_Dir;                          // Richtung festlegen
 PORTA &= ~Zuender_Ein;                         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( (Eingang & Brenner_drin) && (Timerticks < Brenner_Timeout) );

// PORTA |= Zuender_Dir;                         // Richtung umkehren
// DelayUs (500);                                 // Kurze Pause
 Delay (20);
 PORTA |= Zuender_Ein;                          // Motor abschalten

 PORTB &= ~Zwoelf;                              // 12V-Versorgung abschalten

 return (Timerticks < Brenner_Timeout);

}

/*********************************************************
 *
 * Steckt die Mischung an
 *
 ********************************************************/
char Brenner_ein (void)
{
  if (!zuender_ausfahren ())                  // Brenner ausfahren
   {
    PORTB &=~ Ventil;                         // Ventil schliessen
    return (FALSE) ;
   }

  Hochspannung_ein ();                        // Hochspannung einschalten
  Delay (5);
  PORTB |= Ventil;                            // Ventil ffnen
  // Delay (5);
  // Hochspannung_ein ();                        // Hochspannung einschalten

  return (TRUE);
}

/*********************************************************
 *
 * Schaltet den Brenner ab
 *
 ********************************************************/
char Brenner_aus (void)
{
   PORTB &=~Ventil;                          // Ventil schliessen
   Hochspannung_aus ();                      // Hochspanung abschalten
   Delay (50);                              // Kurze Pause bis der Druck abgelassen ist
   return (  zuender_einfahren ());          // Brenner einfahren
}



//--------------------- Motorfunktionen Ende ------------------------------------------------

/*********************************************************
 *
 * Liefert die Warpkernspannung in Millivolt
 *
 ********************************************************/
int AkkuSpannung (void)
{
  long wert;

  wert = ADCH * 256;
  wert += ADCL;
  wert *= 6000;
  wert /= 1024;

  return ((int)wert);
}

/*********************************************************
 *
 * Blinkt bis zum St. Nimmerleinstag
 *
 ********************************************************/
void blinker (void)
{
  while (1)
   {
    // LEDs aus
    PORTC &= ~LED_Ein;
    Delay (20);
    // LEDs ein
    PORTC |= LED_Ein;
    Delay (20);
   }
}

/*********************************************************
 *
 * Fhrt die Motoren in die Nullposition, falls notwendig
 * und gengend Energie zur Verfgung steht
 *
 ********************************************************/
void init_blazer (void)
{
 // Neue Eingangsdaten einladen via I2C
 EingangH = i2c_read (Expander1);
 EingangL = i2c_read (Expander2);
 Eingang = (EingangH << 8) | EingangL;

 // Armgurt auf Haltestrom zurckschalten
 PORTC &= ~Arm_Halten;

 // Auf Absturz infolge der Hochspannung prfen
 Absturzflag = EERead (Adresse_Zuender);
 if (Absturzflag == 0xff) Absturzflag = FALSE;

 Arm_freigeben ();
 zuender_einfahren ();

 Delay (50);

 // Testen, ob die 12V-Versorgung OK ist

 // Kopf runter
 if (!Kopf_senken ()) blinker ();

 // Und wieder rauf
 if (!Kopf_heben ()) blinker ();

 // Batteriespannung messen. Wenn zu gering -> Notbetrieb
 /*
 if (AkkuSpannung () < Minimalenergie)
  {
   Uart_SendString ("\n\rInsuffizientes Energieniveau !!!\r\n");
   blinker ();
  }
 */

 Soundeffekt (1);

}

/***************************************************
 *
 * Macht die Hardware betriebsbereit
 *
 ***************************************************/
void init_devices (void)
{
 CLI ();                        //disable all interrupts
 port_init ();
 timer0_init ();

 MCUCR = 0x00;
 GIMSK = 0x00;
 TIMSK = 0x05;

 timer1_init();
 uart0_init();
 adc_init();

 i2c_init ();

 MCUCR = 0x02;
 GIMSK = 0x40;
 SEI();                         //re-enable interrupts

 init_blazer ();

 if (Absturzflag) Uart_SendString ("\n\rWiedereintritt !\r\n");

}

/***************************************************
 *
 * Zeigt das Einzelfunktionsmen
 *
 ***************************************************/
void zeige_einzelmenue (void)
{
 Uart_SendString ("\r\n\nEinzelfunktionsmenue\n\r");
 Uart_SendString ("--------------------\n\r");

 Uart_SendString ("\r\na Pumpe");
 Uart_SendString ("\r\nb Wischer");
 Uart_SendString ("\r\nc Laser");
 Uart_SendString ("\r\nd Ventil");
 Uart_SendString ("\r\ne Hochspannung");
 Uart_SendString ("\r\nf Lautsprecher");
 Uart_SendString ("\r\ng Kopfheber");
 Uart_SendString ("\r\nh Spannung messen");
 Uart_SendString ("\r\ni Leistungstreiber [1..8]");
 Uart_SendString ("\r\nj Zuender raus");
 Uart_SendString ("\r\nk Zuender rein");

 Uart_SendString ("\r\n\n9 Ende\n\r");
}


/***************************************************
 *
 * Baut das Men fr die Einzelfunktionen auf
 *
 ***************************************************/
void menue_einzelfunktionen (void)
{
 char Tempwert = 0;

 zeige_einzelmenue ();
 while (Taste != '9')
  {
   while (Taste == 0);
   switch (Taste)
    {

     case 'a' : { if (FPumpe == FALSE)
                   {
                    FPumpe = TRUE;
                    PORTB |= Pumpe;
                   } else
                   {
                    FPumpe = FALSE;
                    PORTB &= ~Pumpe;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'b' : { if (FWischer == FALSE)
                   {
                    FWischer = TRUE;
                    PORTA &= ~Reiniger_Ein;
                   } else
                   {
                    FWischer = FALSE;
                    PORTA |= Reiniger_Ein;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'c' : { if (FLaser == FALSE)
                   {
                    FLaser = TRUE;
                    PORTB |= Laser;
                   } else
                   {
                    FLaser = FALSE;
                    PORTB &= ~Laser;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'd' : { if (FVentil == FALSE)
                   {
                    FVentil = TRUE;
                    PORTB |= Ventil;
                   } else
                   {
                    FVentil = FALSE;
                    PORTB &= ~Ventil;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'e' : { if (FZuender == FALSE)
                   {
                    FZuender = TRUE;
                    Hochspannung_ein ();
                   } else
                   {
                    FZuender = FALSE;
                    Hochspannung_aus ();
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'f' : { if (FSpeaker == FALSE)
                   {
                    FSpeaker = TRUE;
                    Sound_ein ();
                   } else
                   {
                    FSpeaker = FALSE;
                    Sound_aus ();
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'


	 case 'g' : { if (FHeber == FALSE)
                   {
                    FHeber = TRUE;
                    PORTB |=Zwoelf;       // 12V-Versorgung einschalten
                    Delay (30);
                    PORTA &= ~Kopf_Ein;
                   } else
                   {
                    FHeber = FALSE;
                    PORTB &= ~Zwoelf;       // 12V-Versorgung einschalten
                    PORTA |= Kopf_Ein;
                   }
                  Taste = 0;
                  break;
    		     } // Von Case 'a'

     case 'h' : {
                  Taste = 0;
                  while (Taste == 0)
                   {
                    Uart_SendString ("Akku : ");
                    UART_SendDec (AkkuSpannung ());
                    Uart_SendString (" mV\r");

                    // 10ms Pause
                    Timerticks = 0;
                    while (Timerticks == 0);
                   }
                  Taste = 0;
                  break;
    		     } // Von Case 'h'

     case 'i' : {
                  Taste = 0;
                  Tempwert = 0;
                  while (Taste != ' ')
                   {

                    while (Taste == 0);

                    // Ausgang 2 "Aux3" bedienen
                    if ( (Taste == '2') && (Tempwert & 2) ) { Tempwert &= ~2; PORTD &=~Aux3; } else
                     if ( (Taste == '2') && !(Tempwert & 2) ) { Tempwert |= 2; PORTD |=Aux3; }

                    // Ausgang 3 "Aux2" bedienen
                    if ( (Taste == '3') && (Tempwert & 4) ) { Tempwert &= ~4; PORTD &=~Aux2; } else
                     if ( (Taste == '3') && !(Tempwert & 4) ) { Tempwert |= 4; PORTD |=Aux2; }

                    // Ausgang 4 "12V" bedienen
                    if ( (Taste == '4') && (Tempwert & 8) ) { Tempwert &= ~8; PORTB &=~Zwoelf; } else
                     if ( (Taste == '4') && !(Tempwert & 8) ) { Tempwert |= 8; PORTB |=Zwoelf; }

                    // Ausgang 5 "Ventil" bedienen
                    if ( (Taste == '5') && (Tempwert & 16) ) { Tempwert &= ~16; PORTB &=~Ventil; } else
                     if ( (Taste == '5') && !(Tempwert & 16) ) { Tempwert |= 16; PORTB |=Ventil; }

                    // Ausgang 6 "Znder" bedienen
                    if ( (Taste == '6') && (Tempwert & 32) ) { Tempwert &= ~32; Hochspannung_aus (); } else
                     if ( (Taste == '6') && !(Tempwert & 32) ) { Tempwert |= 32; Hochspannung_ein (); }

                    // Ausgang 7 "Laser" bedienen
                    if ( (Taste == '7') && (Tempwert & 64) ) { Tempwert &= ~64; PORTB &=~Laser; } else
                     if ( (Taste == '7') && !(Tempwert & 64) ) { Tempwert |= 64; PORTB |=Laser; }

                    // Ausgang 8 "Pumpe" bedienen
                    if ( (Taste == '8') && (Tempwert & 128) )  { Tempwert &= ~128; PORTB &=~Pumpe; } else
                     if ( (Taste == '8') && !(Tempwert & 128) ) { Tempwert |= 128; PORTB |=Pumpe; }

                    // 10ms Pause
                    Delay (1);
                    if (Taste != ' ') Taste = 0;
                   }
                  Taste = 0;
                  break;
    		     } // Von Case 'i'

 case 'j' : {
                  zuender_ausfahren ();
                  Taste = 0;
                  break;
    		     } // Von Case 'j'

 case 'k' : {
                  zuender_einfahren ();
                  Taste = 0;
                  break;
    		     } // Von Case 'k'


    } // Von Switch

   if (Taste != '9') Taste = 0;

  }

 Taste = 0;

}

/***************************************************
 *
 * Zeigt das Hauptmen
 *
 ***************************************************/
void zeige_hauptmenue (void)
{
 Uart_SendString ("\r\n\nHauptmenue\n\r");
 Uart_SendString ("----------\n\r");

 Uart_SendString ("\r\n1 Mundstueck");
 Uart_SendString ("\r\n2 Gurt");
 Uart_SendString ("\r\n3 Kopfheber");
 Uart_SendString ("\r\n4 Wischer");
 Uart_SendString ("\r\n5 Zuender");
 Uart_SendString ("\r\n6 Eingaenge zeigen");
 Uart_SendString ("\r\n7 Einzelfunktionen");
 Uart_SendString ("\r\n8 Funktionen an/abschalten");

 Uart_SendString ("\r\n\n9 Ende\n\r");

}

/***************************************************
 *
 * Baut das Hauptmen auf
 *
 ***************************************************/
void Hauptmenue ()
{

 while (Taste != '9')
  {
   zeige_hauptmenue ();
   while (Taste == 0);

   switch (Taste)
    {

     case '1' : { // Mundstck ein/ausfahren
                  Taste = 0;
                  Uart_SendString ("\r\nMundstueck ");
                  if (KMundsteck) { KMundsteck = FALSE; Erfolg = Mundstueck_einfahren (); } else
                                  { KMundsteck = TRUE; Erfolg = Mundstueck_ausfahren (); }
                  if (Erfolg) Uart_SendString ("OK\n\r");  else  Uart_SendString ("nicht OK\n\r");
                  break;
     			}

     case '2' : { // Gurt ffnen/schliessen
                  Taste = 0;
                  Uart_SendString ("\n\rGurt ");
                  if (KGurt) { Erfolg = Arm_freigeben (); KGurt=FALSE; } else { Erfolg = Arm_spannen (); KGurt = TRUE; }
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;
     			}

     case '3' : { // Kopf ein/ausfahren
                  Taste = 0;
                  Uart_SendString ("\n\rKopf ");
                  if (KKopf) { KKopf=FALSE; Erfolg = Kopf_senken (); } else { KKopf=TRUE; Erfolg = Kopf_heben (); }
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;
                }

     case '4' : { // Wischer bettigen
                  Taste = 0;
                  Uart_SendString ("\n\rWischer ");
                  Erfolg = wischen ();
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;
     			}

     case '5' : { // Mischung anstecken
                  Taste = 0;
                  Uart_SendString ("\n\rBrenner ");
                  if (KBrenner) { Erfolg = Brenner_ein (); KBrenner=FALSE;} else { Erfolg = Brenner_aus (); KBrenner=TRUE; }
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;
     			}

     case '6' : { // Eingangssignale dezimal anzeigen
                  Taste = 0;
                  while (Taste == 0)
                   {
                    Uart_SendString ("Eingang : ");
                    UART_SendDec (Eingang);
                    Uart_SendString ("\r");
                    // 10ms Pause
                    Timerticks = 0;
                    while (Timerticks == 0);
                   }
                  Taste = 0;
                  break;
          		}

     case '7' : { // Einzelfunktionen
                  menue_einzelfunktionen ();
                  Taste = 0;
                  break;
     			}

     case '8' : { // Funktionen ein/ausschalten
                  Taste = 0;
                  break;
     			}

    } // Von Switch

   if (Taste != '9') Taste = 0;

  }  // Von while ()

 Uart_SendString ("\r\nEnde mit dat...");
 Taste = 0;
}

/*************+***************************************************
 *
 * Ja, schiebs mir rein das Bit !!!!!! Mach mir die Maschine !!!
 *
 ****************************************************************/
void Spreng_dir_die_Ruebe_weg (void)
{
 char Eingeschaltet = FALSE;

 // Warten, bis das Ding eingeschaltet wurde
 if (Eingang & Taster_Ein) return;

 // Warte, bis der Idiot seine unegalen Wixgriffel vom Taster gefischt hat....
 while (!(Eingang & Taster_Ein));

 Eingeschaltet = TRUE;

 PORTB &= ~Laser;                     // Laser abschalten

 // Gurt spannen, LED ein
 PORTC &= ~LED_Ein;
 Arm_spannen ();

 // Kopf_heben ();

 // Und ab hier qualmts in der Kiste......
 while (Eingeschaltet)
  {

   // Eine Putzfrau putzt, ein Soldat REINIGT !
   if (!(Eingang & Taster_Reinigen))
    {
     Kopf_senken ();                                    // Kopf senken
     while (!(Eingang & Taster_Reinigen)) wischen ();   // Wischen, solange er mchte
     Kopf_heben ();                                     // Kopf wieder anheben
     Delay (50);                                        // Halbe Sekunde warten
     zuender_einfahren ();                              // Znder wieder reinholen
    }

   // Jetzt macht er sich WEG ! BLAZER, tu dein Schlimmstes !
   if (!(Eingang & Taster_Rauchen))
    {
     PORTB &= ~Laser;                                      // Laser abschalten

     Soundeffekt (4);

     Mundstueck_ausfahren ();                           // Mundstck ausfahren
     // Ist der Taster immer noch unter Begrabbelung ?
     if (Eingang & Taster_Rauchen)
      {
        Mundstueck_einfahren ();                        // Und wieder rein mit dem Rssel
        break;                                          // Und das wars
      }

     // Znder ausfahren und einschalten
     if (!Brenner_ein () )
      {
        Hochspannung_aus ();            // Gefritzel abschalten
        Brenner_aus ();                 // Brenner einfahren
        Mundstueck_einfahren ();        // Und wieder rein mit dem Rssel
        Arm_freigeben ();               // Arm freigeben
        blinker ();                     // Eneergiemangel anzeigen
        break;                          // Und das wars
      }

     // Ist der Taster immer noch unter Begrabbelung ?
     if (Eingang & Taster_Rauchen)
      {
        Hochspannung_aus ();            // Gefritzel abschalten
        Brenner_aus ();                 // Brenner einfahren
        Mundstueck_einfahren ();        // Und wieder rein mit dem Rssel
        break;                          // Und das wars
      }

     // Pumpe einschalten
     Uart_SendString ("\r\nUnd jetzt schoen Atmen !!!! !\r");

     // Ist der Taster immer noch unter Begrabbelung ? Dann Uhr laufen lassen und nach 2 Sekunden
     // die Hochspannung abschalten
     Timerticks = 0;
     while (Timerticks < Hochspannung_Timeout)
      // Ist der Taster immer noch unter Begrabbelung ?
      if (Eingang & Taster_Rauchen)
       {
         PORTB &= ~Pumpe;               // Pumpe abschalten
         Hochspannung_aus ();           // Gefritzel abschalten
         Brenner_aus ();                // Brenner einfahren
         Mundstueck_einfahren ();       // Und wieder rein mit dem Rssel
         break;                         // Und das wars
       }

     // Pumpe einschalten
     PORTB |= Pumpe;

     // Ist der Taster immer noch unter Begrabbelung ? Dann Uhr laufen lassen und nach 2 Sekunden
     // den Brenner abschalten
     Timerticks = 0;
     while (Timerticks < Gas_Timeout)
      // Ist der Taster immer noch unter Begrabbelung ?
      if (Eingang & Taster_Rauchen)
       {
         PORTB &= ~Pumpe;               // Pumpe abschalten
         Brenner_aus ();                // Brenner einfahren
         Mundstueck_einfahren ();       // Und wieder rein mit dem Rssel
         break;                         // Und das wars
       }

     // Brenner wieder einfahren
     Uart_SendString ("\r\nBloss keine Energie verschwenden !\r");
     PORTB &=~Ventil;                   // Ventil schliessen
     Hochspannung_aus ();               // Gefritzel abschalten

     // Wenn Taste losgelassen -> Alles Rckwrts....
     while (!(Eingang & Taster_Rauchen));
     PORTB &= ~Pumpe;                   // Pumpe abschalten
     Mundstueck_einfahren ();           // Und wieder rein mit dem Rssel
     Brenner_aus ();                    // Brenner einfahren

    } // Von if (!(Eingang & Taster_Rauchen))

   if (!(Eingang & Taster_Ein))
    {
     while (!(Eingang & Taster_Ein));
     Eingeschaltet = FALSE;
    }


  }  // Von while (Eingeschaltet)

 // Gurt entspannen, LED aus
 // Achtung : Sie verlassen den zumutbaren Sektor !
  PORTC |= LED_Ein;
  Arm_freigeben ();
  Soundeffekt (2);

}

/***************************************************
 *
 * Hauptprogramm
 *
 ***************************************************/
void main(void)
{

 init_devices ();
 Uart_SendString ("\r\nBlazer (c) 2003 @ Fingers elektrische Welt\r");

 while (1)
  {

    // Wenn gerade keine Aktion stattfindet und die Leertaste gedrckt wurde,
	// Hauptmen einschalten
	if (Taste == ' ')
	 {
	  Taste = 0;
	  Hauptmenue ();
	  Taste = 0;
	 } else Taste = 0;

  // Hauptapplikation
  Spreng_dir_die_Ruebe_weg ();

  // Laser blinken lassen....
  if (Laserblink) PORTB |= Laser; else                 // Laser einschalten
                  PORTB &= ~Laser;                     // Laser abschalten


   // Gas ablassen (Reinigen festhalten)
   if ( (!(Eingang & Taster_Reinigen)) /*&& (!(Eingang & Taster_Rauchen)) */)
    {
      Uart_SendString ("\n\rPfffffff.....\r\n");
      zuender_ausfahren ();
      Delay (20);
      PORTB |= Ventil;                            // Ventil ffnen
      while ( (!(Eingang & Taster_Reinigen)) );
      PORTB &= ~Ventil;                           // Ventil schliessen
      Delay (20);
      zuender_einfahren ();
      Uart_SendString ("\n\rKlick\r\n");
    }

  // Batteriespannung messen. Wenn zu gering -> Notbetrieb
  if (AkkuSpannung () < Minimalenergie)
   {
    Uart_SendString ("\n\rInsuffizientes Energieniveau !!!\r\n");
    while (1)
     {
      // LEDs aus
      PORTC &= ~LED_Ein;
      Delay (500);
      // LEDs ein
      PORTC |= LED_Ein;
      Delay (100);
     }
   }


  } // Von while (1)

}

