/*****************************************************
 *
 * BLAZER (C) Finger @ Fingers elektrische Welt
 *
 * www.fingers-welt.de
 * finger@nwn.de
 *
 * ImageCraft-C
 * AT90S8585 @ 8 MHz
 ****************************************************/
#include <io8535v.h>
#include <macros.h>

// I2C-Leitungen an Port D
#define SDA 0x08
#define SCL 0x10

// Verzgerungszeiten fr I2C
#define uDLY                  40         // Verzgerungszeit zwischen Bitwechseln
#define rDLY                  2          // Verzgerungszeit beim lesen

// Generelle Definitionen
#define FALSE				  0
#define TRUE				  1

// Adressen der I2C-IO-Expander
#define Expander1			  0x70
#define Expander2			  0x72

// Timeouts fr die Motoren *10ms
#define Mundstueck_Timeout    100
#define Gurt_Timeout          200
#define Heber_Timeout         50
#define Wischer_Timeout       100
#define Brenner_Timeout       500

// Eingangsdefinitionen der I2C-Sensorphalanx
#define Mundstueck_drin             0x0001
#define Mundstueck_draussen         0x0002
#define Arm_frei                    0x0004
#define Arm_Kommutierung            0x0008
#define Brenner_drin                0x0010
#define Brenner_draussen            0x0020
#define Kopf_oben                   0x0040
#define Kopf_unten                  0x0080
#define Wischer_drin                0x0100
#define Taster_Reinigen             0x0200
#define Taster_Rauchen              0x0400
#define Taster_Ein                  0x0800

// Ausgangsdefinitionen der IO-Phaserbnke
#define LED_Ein                     0x01    // Port C
#define LED_Rauchen                 0x02    // Port C
#define LED_Reinigen                0x04    // Port C
#define V12_Ein                     0x08    // Port C
#define Mundstueck_Dir              0x10    // Port C
#define Mundstueck_Ein              0x20    // Port C
#define Arm_Dir                     0x40    // Port C
#define Arm_Halten                  0x80    // Port C

#define Arm_Ein                     0x80    // Port A
#define Kopf_Dir                    0x40    // Port A
#define Kopf_Ein                    0x20    // Port A
#define Zuender_Dir                 0x10    // Port A
#define Zuender_Ein                 0x08    // Port A
#define Reiniger_Dir                0x04    // Port A
#define Reiniger_Ein                0x02    // Port A

#define Aux1                        0x01    // Port B
#define Pumpe                       0x02    // Port B
#define Laser                       0x04    // Port B
#define Hochspannung                0x08    // Port B
#define Ventil                      0x10    // Port B

#define Speaker                     0x20    // Port D
#define Aux2                        0x40    // Port D
#define Aux3                        0x80    // Port D

// Variablen
volatile char Taste = 0;							 // Empfangenes Zeichen via RS232
volatile char msTicks = 0;                        // Zhler fr den Timerinterrupt
volatile long Timerticks = 0;					 // Systemuhr
volatile long Delayticks = 0;					 // Fr Warteschleifen
volatile long Spannticks = 0;                    // Zur Kommutierungsdetektion

// Aktuelles Geschehen
volatile unsigned int Eingang = 0;                // Eingangsvariablen
volatile char EingangL;
volatile char EingangH;

// Eingeschaltete Subeinheiten
char FPumpe          = FALSE;
char FVentil         = FALSE;
char FZuender        = FALSE;
char FLaser          = FALSE;
char FWischer        = FALSE;
char FSpeaker        = FALSE;
char FHeber          = FALSE;

// Aktivierte, kognitive Funktionen
char KMundsteck      = FALSE;
char KGurt           = FALSE;
char KKopf           = FALSE;
char KWischer        = FALSE;

// Erfolgsmeldung
char Erfolg          = FALSE;
char I2CAcknowledge  = FALSE;

/***************************************************
 *
 * Macht die Ports klar
 *
 ***************************************************/
void port_init(void)
{                           // DDR = 1 : Ausgang !
 PORTA = 0xFF;
 DDRA  = 0xFE;              // Alles Ausgang, bis auf A0

 PORTB = 0xE0;
 DDRB  = 0x1F;              // 0001.1111

 PORTC = 0xFF;  // 1111.1111 Alles auf High
 DDRC  = 0xFF;  // 1111.1111 Alles auf Ausgang schalten

 PORTD = 0x1F;  // 0001.1111
 DDRD  = 0xF8;  // 1111.1000 Untere 3 auf Ausgang schalten

}

/***************************************************
 *
 * Macht Timer 0 klar, freilaufend mit 100 KHz
 *
 ***************************************************/
void timer0_init(void)
{
 TCCR0 = 0x00;  //stop
 TCNT0 = 0x00;  //set count
 TCCR0 = 0x03;  //start timer, prescaler 64
}

/***************************************************
 *
 * Macht Timer 1 klar, freilaufend
 * PWM an OCR1
 *
 ***************************************************/
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0xD8; //setup
 TCNT1L = 0xF0;
 OCR1AH = 0x27;
 OCR1AL = 0x10;
 OCR1BH = 0x27;
 OCR1BL = 0x10;
 TCCR1A = 0x81;
 TCCR1B = 0x02; //start Timer
}

/***************************************************
 *
 * Macht den UART klar, 38400 BPS
 *
 ***************************************************/
void uart0_init(void)
{
 UCR  = 0x00; //disable while setting baud rate
 UBRR = 0x0C; //set baud rate 38400 @ 8 MHz
 UCR  = 0xD8; //enable
}

/***************************************************
 *
 * Timer-0 ISR
 * Erhht alle 10 ms den Systemticker
 *
 ***************************************************/
 #pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
 TCNT0 = 0x00; 			   					//reload counter value
 msTicks++;
 if (msTicks == 10)
  {
   Timerticks++; 						 		// Uhr hochzhlen
   msTicks=0;
  }
}

/***************************************************
 *
 * UART-Receive ISR
 *
 ***************************************************/
#pragma interrupt_handler uart0_rx_isr:12
void uart0_rx_isr(void)
{
 char Junk;

 // Empfangenes Byte verwerfen, wenn das letzte noch nicht verwurstet wurde
 if (Taste != 0) Junk = UDR; else Taste = UDR;
}

/***************************************************
 *
 * UART-Transmit-ISR
 *
 ***************************************************/
#pragma interrupt_handler uart0_tx_isr:14
void uart0_tx_isr(void)
{
}

/***************************************************
 *
 * Macht den ADC klar
 *
 ***************************************************/
void adc_init(void)
{
 ADCSR = 0x00; //disable adc
 ADMUX = 0x00; //select adc input 0
 ADCSR = 0xE1;
}

/***************************************************
 *
 * Sendet ein Byte via UART
 *
 ***************************************************/
void UART_SendByte (char Data)
{
    while(!(USR & 0x20));
    UDR=Data;  // Und raus mit dem Scheiss
}

/***************************************************
 *
 * Sendet eine Zeichenkette via UART
 *
 ***************************************************/
void Uart_SendString (const char* Data)
{
  do
   UART_SendByte (*(Data++));
  while (*Data != '\0');
}

/***************************************************
 *
 * Sendet eine Zahl im Klartext via UART
 *
 ***************************************************/
void UART_SendDec (unsigned int Data)
{
      char         c;
      unsigned int i;
      char         flag=0;

      UART_SendByte (0x20);

      for (i=10000;i!=0;i/=10)
      {
          c=Data/i;
          Data%=i;
          if(c!=0||flag==1||i==1)
          {
              flag=1;
              c+=0x30;
              UART_SendByte (c);
          }
          if(flag==0 && i<=100)
          {
              UART_SendByte (0x20);
          }
      }
}

/***************************************************
 *
 * Liest ein Byte aus dem EEPROM
 *
 ***************************************************/
char EERead(char address)
{
    while (EECR & 0x02);
    EEAR = address;
    EECR = 0x01;
    return EEDR;
}

/***************************************************
 *
 * Schreibt ein Byte ins EEPROM
 *
 ***************************************************/
void EEWrite (char address, char data)
{
    while (EECR & 0x02);
    EEAR = address;
    EEDR = data;
    EECR = 0x04;
    EECR = 0x02;
}

/***************************************************
 *
 * Wartet Ticks * 10ms
 *
 ***************************************************/
void Delay (unsigned int Ticks)
  {
   Delayticks = Timerticks;
   while ( (Delayticks+Ticks) > Timerticks);
  }

/******************************
 *
 * Wartet einige Mikrosekunden
 *
 ******************************/
void DelayUs (int wert)
{
 int z,i = 0;
 for (i=0; i<wert; i++) z = i+1;
}

/************************
 *
 * Schaltet SDA auf High
 *
 ************************/
void SDA_High (void)
{
  PORTD |= SDA;																										// SDA = 1;
}

/************************
 *
 * Schaltet SDA auf Low
 *
 ************************/
void SDA_Low (void)
{
  PORTD &= ~SDA;																										// SDA = 0;
}

/************************
 *
 * Schaltet SDA auf High
 *
 ************************/
void SCL_High (void)
{
  PORTD |= SCL;																										// SCL = 1;
}

/************************
 *
 * Schaltet SCL auf Low
 *
 ************************/
void SCL_Low (void)
{
  PORTD &= ~SCL;																										// SCL = 0;
}

/***********************************************************************************
 *
 * Schaltet SDA und SCL als Ausgang und setzt beide auf 1
 *
 ***********************************************************************************/
void i2c_init (void)
 {
  DDRD |= SDA;																										// PortD.3 als Ausgang (1)
  DDRD |= SCL;																										// PortD.4 als Ausgang (1)

  SDA_High ();
  SCL_High ();
 }

/*************************************************************************************
 *
 * Schreibt ein Byte via i2C, erzeugt kein Start/Stop, wertet ACK vom Slave nicht aus
 *
 *************************************************************************************/
char i2c_writebyte (unsigned char tbyte)
 {
  unsigned char i;
  char Ack = FALSE;

  for (i=0; i<8; i++)
   {
     if (tbyte>127) SDA_High (); else SDA_Low ();                           // SDA bedienen
     DelayUs (uDLY);                                            // kurze Pause
     SCL_High (); DelayUs (uDLY);                                     // SCL = 1 + Pause
     SCL_Low (); DelayUs (uDLY);                                     // SCL = 0 + Pause
     tbyte <<= 1;
    }

  // 9. Bit erzeugen, damit der Slave das Acknowledge loswerden kann
  DDRD &= ~SDA;														// PortD.3 als Eingang (0)
  PORTD &= ~SDA;													// Pullup abschalten

  SCL_High ();DelayUs (uDLY);                                      // SCL = 1

  Ack = !(PIND & SDA);                                             // Acknowledge reinholen

  SCL_Low (); DelayUs (uDLY);                                      // SCL = 0 + Pause um ACK abzuwarten

  SCL_Low ();                                                      // SCL = 0

  DDRD |= SDA;													   // PortD.3 als Ausgang (1)

  return (Ack);

 }

/***********************************************************************************
 *
 * Liest ein Byte via i2C, erzeugt bei Bedarf ein Acknowledge
 *
 ***********************************************************************************/
unsigned char i2c_readbyte (char Acknowledge)
 {
  unsigned char i, tbyte = 0;

  DDRD &= ~SDA;																										// PortD.3 als Eingang (0)

  for (i=0; i<8; i++)                                             // Bits rausschieben
   {
     tbyte <<= 1;
     if (PIND & SDA) tbyte |= 1;
     DelayUs (rDLY);
     SCL_High (); DelayUs (rDLY);
     SCL_Low (); DelayUs (rDLY);
   }

  // 9. Bit erzeugen
  DDRD |= SDA;																										// PortD.3 als Ausgang (1)

  if (Acknowledge) SDA_Low (); else SDA_High ();
  DelayUs (rDLY);                                                  // Pause
  SCL_High (); DelayUs (rDLY);                                           // SCL = 1 + Pause
  SCL_Low (); DelayUs (rDLY);                                           // SCL = 0 + Pause
  SDA_High ();DelayUs (rDLY);                                            // SDA = 1 + Pause

  return (tbyte);
}

/***********************************************************************************
 *
 * Liest ein Byte via i2C an Adresse "adress", erzeugt Start/Stop
 *
 ***********************************************************************************/
unsigned char i2c_read (unsigned char deviceaddr)
 {
  unsigned char tbyte;
  char Ack = FALSE;

  // Start-Condition erzeugen
  SDA_High (); SCL_High (); DelayUs (rDLY);                      // SDA + SCL = 1 + Pause
  SDA_Low (); DelayUs (rDLY);                                    // SDA = 0 + Pause
  SCL_Low (); DelayUs (rDLY);                                    // SCL = 0 + Pause

  // Slave zum Lesen adressieren
  tbyte = deviceaddr+1;                                          // Leseadresse setzen
  Ack = i2c_writebyte (tbyte);                                   // Adresse senden

  tbyte = i2c_readbyte (FALSE);                                  // Ein Byte lesen

  // Stop-Condition erzeugen
  SDA_Low (); DelayUs (rDLY);                                    // SDA = 0 + Pause
  SCL_High (); DelayUs (rDLY);                                   // SCL = 1 + Pause
  SDA_High (); DelayUs (rDLY);                                   // SCL = 1 + Pause

  I2CAcknowledge = Ack;

  return (tbyte);
 }

/***************************************************
 *
 * Int0-ISR mit fallender Flanke
 *
 ***************************************************/
#pragma interrupt_handler int0_isr:2
void int0_isr (void)
{
 // Neue Eingangsdaten einladen via I2C
 EingangH = i2c_read (Expander1);
 EingangL = i2c_read (Expander2);
 Eingang = (EingangH << 8) | EingangL;
}


//--------------------- Motorfunktionen -----------------------------------------------------

/***************************************************
 *
 * Fhrt das Mundstck aus (TRUE wenn alles OK)
 *
 ***************************************************/
char Mundstueck_ausfahren (void)
{
 // Isses schon draussen ?
 if (Eingang & Mundstueck_draussen) return (TRUE);

 // Offensichtlich nicht......
 PORTC |= Mundstueck_Dir;                   // Richtung festlegen
 PORTC &= ~Mundstueck_Ein;                  // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Mundstueck_draussen) && (Timerticks < Mundstueck_Timeout) );
 PORTC |= Mundstueck_Ein;                  // Motor abschalten

 return (Timerticks < Mundstueck_Timeout);
}

/***************************************************
 *
 * Fhrt das Mundstck ein (TRUE wenn alles OK)
 *
 ***************************************************/
char Mundstueck_einfahren (void)
{
 // Isses schon draussen ?
 if (Eingang & Mundstueck_drin) return (TRUE);

 // Offensichtlich nicht......
 PORTC &= ~Mundstueck_Dir;                   // Richtung festlegen
 PORTC &= ~Mundstueck_Ein;                  // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Mundstueck_drin) && (Timerticks < Mundstueck_Timeout) );
 PORTC |= Mundstueck_Ein;                  // Motor abschalten

 return (Timerticks < Mundstueck_Timeout);
}

/***************************************************
 *
 * Hlt den Arm des Delinquenten fest
 *
 ***************************************************/
char Arm_spannen (void)
{
 unsigned int TempEingang = 0;
 char Fertig = FALSE;

 #define Spanntimeout 20

 PORTC |= Arm_Dir;          // Richtung festlegen
 PORTC |= Arm_Halten;       // Fahrstrom einschalten
 PORTC &= ~Arm_Ein;         // Motor einschalten

 // Warten, bis keine Kommutierungsimpulse mehr kommen oder die Zeit berschritten wurde
 Timerticks = 0;
 Delayticks = 0;
 Spannticks = 0;

 TempEingang = (Eingang & Arm_Kommutierung);

 while (!Fertig && (Timerticks < Gurt_Timeout) )
  {

   if ( TempEingang != (Eingang & Arm_Kommutierung) )
    {
     // Signal hat sich gendert
     TempEingang = (Eingang & Arm_Kommutierung);
     Spannticks = 0;
    } else
    { // Signal hat sich nicht gendert
      if (Timerticks != Delayticks)
       {
        Delayticks = Timerticks;
        Spannticks++;
        if (Spannticks > Spanntimeout) Fertig = TRUE;
       }
    }

  } // Von While....

 PORTC &= ~Arm_Halten;                   // Auf Haltestrom zurckschalten
 if (!Fertig)  PORTC |= Arm_Ein;         // Motor abschalten im Fehlerfall

 return (Fertig);
}


/***************************************************
 *
 * Gibt den Arm des Delinquenten frei
 *
 ***************************************************/
char Arm_freigeben (void)
{

 // Arm schon frei ?
 if (Eingang & Arm_frei) return (TRUE);

 PORTC &= ~Arm_Dir;         // Richtung festlegen
 PORTC |= Arm_Halten;       // Fahrstrom einschalten
 PORTC &= ~Arm_Ein;         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Arm_frei) && (Timerticks < Gurt_Timeout) );

 PORTC &= ~Arm_Halten;      // Auf Haltestrom zurckschalten
 PORTC |= Arm_Ein;          // Motor abschalten

 return (Timerticks < Gurt_Timeout);

}

/***************************************************
 *
 * Senkt den Kopf ab
 *
 ***************************************************/
char Kopf_senken (void)
{

 // Iss der kleine Scheisser schon unten ?
 if (Eingang & Kopf_unten) return (TRUE);

 PORTC |= V12_Ein;    // 12V-Versorgung einschalten
 PORTA |= Kopf_Dir;   // Richtung festlegen
 PORTA &= ~Kopf_Ein;   // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Kopf_unten) && (Timerticks < Heber_Timeout) );

 PORTC &= ~V12_Ein;    // 12V-Versorgung abschalten
 PORTA |= Kopf_Ein;     // Motor abschalten

 return (Timerticks < Heber_Timeout);

}

/***************************************************
 *
 * Hebt den Kopf an
 *
 ***************************************************/
char Kopf_heben (void)
{

 // Iss der kleine Scheisser schon oben ?
 if (Eingang & Kopf_oben) return (TRUE);

 PORTC |= V12_Ein;    // 12V-Versorgung einschalten
 PORTA &= ~Kopf_Dir;   // Richtung festlegen
 PORTA &= ~Kopf_Ein;   // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Kopf_oben) && (Timerticks < Heber_Timeout) );

 PORTC &= ~V12_Ein;    // 12V-Versorgung abschalten
 PORTA |= Kopf_Ein;     // Motor abschalten

 return (Timerticks < Heber_Timeout);

}

/***************************************************
 *
 * Reinigt den Kopf
 *
 ***************************************************/
char wischen (void)
{
   PORTA |= Reiniger_Dir;       // Richtung festlegen
   PORTA &= ~Reiniger_Ein;       // Wischer einschalten

   // Ist der Sensor aktiv ?
   Timerticks = 0;
   if (Eingang & Wischer_drin)
   while ( (Eingang & Wischer_drin) && (Timerticks < Wischer_Timeout) );
   if (Timerticks >= Wischer_Timeout)
    {
     PORTA |= Reiniger_Ein;       // Wischer abschalten
     return (FALSE);
    }

   // Warten, bis er wieder drin ist
   Timerticks = 0;
   while ( !(Eingang & Wischer_drin) && (Timerticks < Wischer_Timeout) );

   return (Timerticks < Wischer_Timeout);

}

/***************************************************
 *
 * Fhrt den Znder aus
 *
 ***************************************************/
char zuender_ausfahren (void)
{

 // Isses schon draussen ?
 if (Eingang & Brenner_draussen) return (TRUE);

 // Offensichtlich nicht......
 PORTA &= ~Zuender_Dir;                          // Richtung festlegen
 PORTA &= ~Zuender_Ein;                         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Brenner_draussen) && (Timerticks < Brenner_Timeout) );
 PORTA |= Zuender_Ein;                          // Motor abschalten

 return (Timerticks < Brenner_Timeout);

}

/***************************************************
 *
 * Fhrt den Znder ein
 *
 ***************************************************/
char zuender_einfahren (void)
{
 // Isses schon drin ?
 if (Eingang & Brenner_drin) return (TRUE);

 // Offensichtlich nicht......
 PORTA |= Zuender_Dir;                          // Richtung festlegen
 PORTA &= ~Zuender_Ein;                         // Motor einschalten

 // Warten, bis Position erreicht ist oder die Zeit berschritten wurde
 Timerticks = 0;
 while ( !(Eingang & Brenner_drin) && (Timerticks < Brenner_Timeout) );
 PORTA |= Zuender_Ein;                          // Motor abschalten

 return (Timerticks < Brenner_Timeout);

}


//--------------------- Motorfunktionen Ende ------------------------------------------------

/*********************************************************
 *
 * Fhrt die Motoren in die Nullposition, falls notwendig
 * und gengend Energie zur Verfgung steht
 *
 ********************************************************/
void init_blazer (void)
{
 // Neue Eingangsdaten einladen via I2C
 EingangH = i2c_read (Expander1);
 EingangL = i2c_read (Expander2);
 Eingang = (EingangH << 8) | EingangL;
}

/***************************************************
 *
 * Macht die Hardware betriebsbereit
 *
 ***************************************************/
void init_devices (void)
{
 CLI ();                        //disable all interrupts
 port_init ();
 timer0_init ();

 MCUCR = 0x00;
 GIMSK = 0x00;
 TIMSK = 0x01;

 timer1_init();
 uart0_init();
 adc_init();

 i2c_init ();

 MCUCR = 0x02;
 GIMSK = 0x40;
 SEI();                         //re-enable interrupts

 init_blazer ();
}



/***************************************************
 *
 * Zeigt das Einzelfunktionsmen
 *
 ***************************************************/
void zeige_einzelmenue (void)
{
 Uart_SendString ("\r\n\nEinzelfunktionsmenue\n\r");
 Uart_SendString ("--------------------\n\r");

 Uart_SendString ("\r\na Pumpe");
 Uart_SendString ("\r\nb Wischer");
 Uart_SendString ("\r\nc Laser");
 Uart_SendString ("\r\nd Ventil");
 Uart_SendString ("\r\ne Hochspannung");
 Uart_SendString ("\r\nf Lautsprecher");
 Uart_SendString ("\r\ng Kopfheber");

 Uart_SendString ("\r\n\n9 Ende\n\r");
}


/***************************************************
 *
 * Baut das Men fr die Einzelfunktionen auf
 *
 ***************************************************/
void menue_einzelfunktionen (void)
{
 zeige_einzelmenue ();
 while (Taste != '9')
  {
   while (Taste == 0);
   switch (Taste)
    {

     case 'a' : { if (FPumpe == FALSE)
                   {
                    FPumpe = TRUE;
                    PORTB |= Pumpe;
                   } else
                   {
                    FPumpe = FALSE;
                    PORTB &= ~Pumpe;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'b' : { if (FWischer == FALSE)
                   {
                    FWischer = TRUE;
                    PORTA &= ~Reiniger_Ein;
                   } else
                   {
                    FWischer = FALSE;
                    PORTA |= Reiniger_Ein;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'c' : { if (FLaser == FALSE)
                   {
                    FLaser = TRUE;
                    PORTB |= Laser;
                   } else
                   {
                    FLaser = FALSE;
                    PORTB &= ~Laser;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'd' : { if (FVentil == FALSE)
                   {
                    FVentil = TRUE;
                    PORTB |= Ventil;
                   } else
                   {
                    FVentil = FALSE;
                    PORTB &= ~Ventil;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'e' : { if (FZuender == FALSE)
                   {
                    FZuender = TRUE;
                    PORTB |= Hochspannung;
                   } else
                   {
                    FZuender = FALSE;
                    PORTB &= ~Hochspannung;
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'f' : { if (FSpeaker == FALSE)
                   {
                    FSpeaker = TRUE;
                    // Timerwert zuweisen
                   } else
                   {
                    FSpeaker = FALSE;
                    // Timerwert zuweisen
                   }
                  Taste = 0;
                  break;
     		     } // Von Case 'a'

     case 'g' : { if (FHeber == FALSE)
                   {
                    FHeber = TRUE;
                    PORTC |= V12_Ein;     // 12V-Versorgung einschalten
                    PORTA &= ~Kopf_Ein;
                   } else
                   {
                    FHeber = FALSE;
                    PORTC &= ~V12_Ein;    // 12V-Versorgung abschalten
                    PORTA |= Kopf_Ein;
                   }
                  Taste = 0;
                  break;
    		     } // Von Case 'a'


    } // Von Switch

   if (Taste != '9') Taste = 0;

  }

 Taste = 0;

}

/***************************************************
 *
 * Zeigt das Hauptmen
 *
 ***************************************************/
void zeige_hauptmenue (void)
{
 Uart_SendString ("\r\n\nHauptmenue\n\r");
 Uart_SendString ("----------\n\r");

 Uart_SendString ("\r\n1 Mundstueck");
 Uart_SendString ("\r\n2 Gurt");
 Uart_SendString ("\r\n3 Kopfheber");
 Uart_SendString ("\r\n4 Wischer");
 Uart_SendString ("\r\n5 Zuender");
 Uart_SendString ("\r\n6 Eingaenge zeigen");
 Uart_SendString ("\r\n7 Einzelfunktionen");
 Uart_SendString ("\r\n8 Funktionen an/abschalten");

 Uart_SendString ("\r\n\n9 Ende\n\r");

}

/***************************************************
 *
 * Baut das Hauptmen auf
 *
 ***************************************************/
void Hauptmenue ()
{

 while (Taste != '9')
  {
   zeige_hauptmenue ();
   while (Taste == 0);

   switch (Taste)
    {

     case '1' : { // Mundstck ein/ausfahren
                  Taste = 0;
                  Uart_SendString ("\r\nMundstueck ");
                  if (KMundsteck) Erfolg = Mundstueck_einfahren (); else Erfolg = Mundstueck_ausfahren ();
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;

     			}

     case '2' : { // Gurt ffnen/schliessen
                  Taste = 0;
                  Uart_SendString ("\n\rGurt ");
                  if (KGurt) Erfolg = Arm_freigeben (); else Erfolg = Arm_spannen ();
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;

     			}

     case '3' : { // Kopf ein/ausfahren
                  Taste = 0;
                  Uart_SendString ("\n\rKopf ");
                  if (KKopf) Erfolg = Kopf_senken (); else Erfolg = Kopf_heben ();
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;

                }

     case '4' : { // Wischer bettigen
                  Taste = 0;
                  Uart_SendString ("\n\rWischer ");
                  Erfolg = wischen ();
                  if (Erfolg) Uart_SendString ("OK\n\r"); else Uart_SendString ("nicht OK\n\r");
                  break;

     			}

     case '5' : { // Mischung anstecken
                  Taste = 0;
                  break;

     			}

     case '6' : { // Eingangssignale
                  Taste = 0;
                  Taste = 0;
                  while (Taste == 0)
                   {
                    Uart_SendString ("Eingang : ");
                    UART_SendDec (Eingang);
                    Uart_SendString ("\r");
                    // 10ms Pause
                    Timerticks = 0;
                    while (Timerticks == 0);
                   }
                  break;

          			}


     case '7' : { // Einzelfunktionen
                  menue_einzelfunktionen ();
                  Taste = 0;
                  break;

     			      }

     case '8' : { // Funktionen ein/ausschalten
                  Taste = 0;
                  break;

     			      }

    } // Von Switch

   if (Taste != '9') Taste = 0;

  }  // Von while ()

 Uart_SendString ("\r\nEnde mit dat...");
 Taste = 0;
}

/***************************************************
 *
 * Hauptprogramm
 *
 ***************************************************/
void main(void)
{

 init_devices ();
 Uart_SendString ("\r\nBlazer (c) 2003 @ Fingers elektrische Welt\r");

 while (1)
  {

    // Wenn gerade keine Aktion stattfindet und die Leertaste gedrckt wurde,
	// Hauptmen einschalten
	if (Taste == ' ')
	 {
	  Taste = 0;
	  Hauptmenue ();
	  Taste = 0;
	 } else Taste = 0;

  // Hauptapplikation


  } // Von while (1)

}

